/* Break.java

	Purpose:
		
	Description:
		
	History:
		Thu, Oct  8, 2015 10:34:52 AM, Created by Christopher

Copyright (C) 2015 Potix Corporation. All Rights Reserved.

This program is distributed under LGPL Version 2.1 in the hope that
it will be useful, but WITHOUT ANY WARRANTY.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * Represents a break in the axis. The sections defined will be left out and all the points shifted closer to each other.
 * @author Christopher
 */
public class Break extends Optionable{
	private static final long serialVersionUID = -3590346971432941889L;
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		breakSize,
		from,
		repeat,
		to
	}
	
	/**
	 * A number indicating how much space should be left between the start and
	 * the end of the break. The break size is given in axis units, so for
	 * instance on a datetime axis, a break size of 3600000 would indicate the
	 * equivalent of an hour. Defaults to 0.
	 * 
	 * @return the break value
	 */
	public Number getBreakSize() {
		return getAttr(Attrs.breakSize, 0).asNumber();
	}
	/**
	 * Set the break size.
	 * @param breakSize
	 * @see #getBreakSize()
	 */
	public void setBreakSize(Number breakSize) {
		setAttr(Attrs.breakSize, breakSize, 0);
	}
	
	/**
	 * The point where the break starts.
	 * @return the starting value of this break
	 */
	public Number getFrom() {
		return getAttr(Attrs.from, null).asNumber();
	}
	/**
	 * Set the starting value of this break
	 * @param from
	 */
	public void setFrom(Number from) {
		setAttr(Attrs.from, from);
	}
	
	/**
	 * Defines an interval after which the break appears again. By default the
	 * breaks do not repeat.<p>
	 * If repeat value is greater than 0, the next break will appear at {@link #getFrom() from} + repeat, and so on.
	 * <p>
	 * For example: a break with from = 5, to = 10, and repeat = 10. The next break will appear at from = 15 and to = 20, and so on so forth.
	 * <p>
	 * Defaults to 0.
	 * @return the repeat value
	 */
	public Number getRepeat() {
		return getAttr(Attrs.repeat, 0).asNumber();
	}
	/**
	 * Set the repeat value for this break.
	 * @param Repeat
	 */
	public void setRepeat(Number Repeat) {
		setAttr(Attrs.repeat, Repeat, 0);
	}
	
	/**
	 * The point where the break ends.
	 * @return the ending value of this break
	 */
	public Number getTo() {
		return getAttr(Attrs.to, null).asNumber();
	}
	/**
	 * Set the ending value of this break
	 * @param to
	 */
	public void setTo(Number to) {
		setAttr(Attrs.to, to);
	}
}
