/* Level.java

	Purpose:
		
	Description:
		
	History:
		Sat, Oct  3, 2015  3:32:38 PM, Created by Christopher

Copyright (C) 2015 Potix Corporation. All Rights Reserved.

This program is distributed under LGPL Version 2.1 in the hope that
it will be useful, but WITHOUT ANY WARRANTY.
*/
package org.zkoss.chart;

import org.zkoss.chart.plotOptions.DataLabels;
import org.zkoss.chart.util.DynamicalAttribute;

/**
 * Set options on specific levels.
 * Takes precedence over series options, but not point options.
 * 
 * @author Christopher
 */
public class Level extends Optionable{
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		borderColor,
		borderDashStyle,
		borderWidth,
		color,
		dataLabels,
		layoutAlgorithm,
		layoutStartingDirection,
		level
	}

	/**
	 * Set the borderColor on all points which lies on the same level.
	 * <p>
	 * Default: null
	 * @since 2.1.0
	 */
	public Color getBorderColor() {
		return (Color) getAttr(Attrs.borderColor, null).asValue();
	}
	/**
	 * Set the borderColor on all points which lies on the same level.
	 * @param color
	 * @since 2.1.0
	 */
	public void setBorderColor(Color color) {
		setAttr(Attrs.borderColor, color, NOT_NULL_VALUE);
	}
	/**
	 * Set the borderColor on all points which lies on the same level.
	 * @param color
	 * @since 2.1.0
	 */
	public void setBorderColor(String color) {
		setBorderColor(new Color(color));
	}
	/**
	 * Set the borderColor on all points which lies on the same level.
	 * @param color
	 * @since 2.1.0
	 */
	public void setBorderColor(LinearGradient color) {
		setBorderColor(new Color(color));
	}
	/**
	 * Set the borderColor on all points which lies on the same level.
	 * @param color
	 * @since 2.1.0
	 */
	public void setBorderColor(RadialGradient color) {
		setBorderColor(new Color(color));
	}

	/**
	 * Returns a name for the dash style to use for the graph. Applies only to
	 * series type having a graph, like <code>line</code>, <code>spline</code>,
	 * <code>area</code> and <code>scatter</code> in case it has a
	 * <code>lineWidth</code>. The value for the <code>dashStyle</code> include:
	 * <ul>
	 * <li>Solid</li>
	 * <li>ShortDash</li>
	 * <li>ShortDot</li>
	 * <li>ShortDashDot</li>
	 * <li>ShortDashDotDot</li>
	 * <li>Dot</li>
	 * <li>Dash</li>
	 * <li>LongDash</li>
	 * <li>DashDot</li>
	 * <li>LongDashDot</li>
	 * <li>LongDashDotDot</li>
	 * </ul>
	 * @since 2.1.0
	 */
	public String getBorderDashStyle() {
		return getAttr(Attrs.borderDashStyle, null).asString();
	}
	/**
	 * Set the dash style for the border.
	 * @param borderDashStyle
	 * @see #getBorderDashStyle()
	 * @since 2.1.0
	 */
	public void setBorderDashStyle(String borderDashStyle) {
		if (!("Solid".equalsIgnoreCase(borderDashStyle)
				|| "ShortDash".equalsIgnoreCase(borderDashStyle)
				|| "ShortDot".equalsIgnoreCase(borderDashStyle)
				|| "ShortDashDot".equalsIgnoreCase(borderDashStyle)
				|| "ShortDashDotDot".equalsIgnoreCase(borderDashStyle)
				|| "Dot".equalsIgnoreCase(borderDashStyle)
				|| "Dash".equalsIgnoreCase(borderDashStyle)
				|| "LongDash".equalsIgnoreCase(borderDashStyle)
				|| "DashDot".equalsIgnoreCase(borderDashStyle)
				|| "LongDashDot".equalsIgnoreCase(borderDashStyle)
				|| "LongDashDotDot".equalsIgnoreCase(borderDashStyle)))
			throw new IllegalArgumentException("Unsupported style: ["
					+ borderDashStyle + "]");
		setAttr(Attrs.borderDashStyle, borderDashStyle, "Solid");
	}

	/**
	 * Get the width of the border.
	 * <p>
	 * Default: null
	 * @since 2.1.0
	 */
	public Number getBorderWidth() {
		return getAttr(Attrs.borderWidth, null).asNumber();
	}
	/**
	 * Set the width of the border.
	 * @param borderWidth
	 * @since 2.1.0
	 */
	public void setBorderWidth(Number borderWidth) {
		setAttr(Attrs.borderWidth, borderWidth);
	}

	/**
	 * Get the color on all points which lies on the same level.
	 * <p>
	 * Default: null
	 * @since 2.1.0
	 */
	public Color getColor() {
		return (Color) getAttr(Attrs.color, null).asValue();
	}
	/**
	 * Set the color on all points which lies on the same level.
	 * @since 2.1.0
	 */
	public void setColor(Color color) {
		setAttr(Attrs.color, color);
	}
	/**
	 * Set the color on all points which lies on the same level.
	 * @since 2.1.0
	 */
	public void setColor(String color) {
		setColor(new Color(color));
	}
	/**
	 * Set the color on all points which lies on the same level.
	 * @since 2.1.0
	 */
	public void setColor(LinearGradient color) {
		setColor(new Color(color));
	}
	/**
	 * Set the color on all points which lies on the same level.
	 * @since 2.1.0
	 */
	public void setColor(RadialGradient color) {
		setColor(new Color(color));
	}

	/**
	 * Get the options of dataLabels on each point which lies on the level.
	 * <p>
	 * Default: null
	 * @since 2.1.0
	 */
	public DataLabels getDataLabels() {
		return (DataLabels) this.getAttr(Attrs.dataLabels, null).asValue();
	}
	/**
	 * Set the options of dataLabels on each point which lies on the level.
	 * @since 2.1.0
	 */
	public void setDataLabels(DataLabels dataLabels) {
		setAttr(Attrs.dataLabels, dataLabels);
	}

	/**
	 * Get the layoutAlgorithm option on a specific level.
	 * <p>
	 * Default: null
	 * @since 2.1.0
	 */
	public String getLayoutAlgorithm() {
		return getAttr(Attrs.layoutAlgorithm, null).asString();
	}
	/**
	 * Set the layoutAlgorithm option on a specific level.
	 * @since 2.1.0
	 */
	public void setLayoutAlgorithm(String layoutAlgorithm) {
		setAttr(Attrs.layoutAlgorithm, layoutAlgorithm);
	}

	/**
	 * Get the layoutStartingDirection option on a specific level.
	 * <p>
	 * Default: null
	 * @since 2.1.0
	 */
	public String getLayoutStartingDirection() {
		return getAttr(Attrs.layoutStartingDirection, null).asString();
	}
	/**
	 * Set the layoutStartingDirection option on a specific level.
	 * @since 2.1.0
	 */
	public void setLayoutStartingDirection(String layoutStartingDirection) {
		setAttr(Attrs.layoutStartingDirection, layoutStartingDirection);
	}

	/**
	 * Decides which level takes effect from the options set in the levels object.
	 * <p>
	 * Default: null
	 * @since 2.1.0
	 */
	public Number getLevel() {
		return getAttr(Attrs.level, null).asNumber();
	}
	/**
	 * Set which level takes effect from the options set in the levels object.
	 * @since 2.1.0
	 */
	public void setLevel(Number level) {
		setAttr(Attrs.level, level);
	}
}
