package org.zkoss.chart;


/**
 * a utility to get Highcharts theme default color sets
 * 
 * @author hanhsu
 */
public class ThemeColors {
	
	private ThemeColors() {
	} // prevent from creation 
	
	/**
	 * The default color set for the default theme by highcharts.
	 */
	private static final String[] DEFAULT_COLORS = {"#7cb5ec", "#434348",
		"#90ed7d", "#f7a35c", "#8085e9", "#f15c80", "#e4d354", "#8085e8", "#8d4653", "#91e8e1"};
	/**
	 * The default color set for the grid theme by highcharts.
	 */
	private static final String[] GRID_COLORS = {"#058DC7", "#50B432",
		"#ED561B", "#DDDF00", "#24CBE5", "#64E572", "#FF9655", "#FFF263", "#6AF9C4"};
	/**
	 * The default color set for the skies theme by highcharts.
	 */
	private static final String[] SKIES_COLORS = {"#514F78", "#42A07B",
		"#9B5E4A", "#72727F", "#1F949A", "#82914E", "#86777F", "#42A07B"};
	/**
	 * The default color set for the gray theme by highcharts.
	 */
	private static final String[] GRAY_COLORS = {"#DDDF0D", "#7798BF",
		"#55BF3B", "#DF5353", "#aaeeee", "#ff0066", "#eeaaee", "#55BF3B", "#DF5353", "#7798BF", "#aaeeee"};
	/**
	 * The default color set for the dark blue theme by highcharts.
	 */
	private static final String[] DARK_BLUE_COLORS = {"#DDDF0D", "#55BF3B",
		"#DF5353", "#7798BF", "#aaeeee", "#ff0066", "#eeaaee", "#55BF3B", "#DF5353", "#7798BF", "#aaeeee"};
	/**
	 * The default color set for the dark green theme by highcharts.
	 */
	private static final String[] DARK_GREEN_COLORS = {"#DDDF0D", "#55BF3B",
		"#DF5353", "#7798BF", "#aaeeee", "#ff0066", "#eeaaee", "#55BF3B", "#DF5353", "#7798BF", "#aaeeee"};
	/**
	 * The default color set for the dark unica theme by highcharts.
	 */
	private static final String[] DARK_UNICA_COLORS = {"#2b908f", "#90ee7e",
		"#f45b5b", "#7798BF", "#aaeeee", "#ff0066", "#eeaaee", "#55BF3B", "#DF5353", "#7798BF", "#aaeeee"};
	/**
	 * The default color set for the sand signika theme by highcharts.
	 */
	private static final String[] SAND_SIGNIKA_COLORS = {"#f45b5b", "#8085e9",
		"#8d4654", "#7798BF", "#aaeeee", "#ff0066", "#eeaaee", "#55BF3B", "#DF5353", "#7798BF", "#aaeeee"};
	/**
	 * The default color set for the grid light theme by highcharts.
	 */
	private static final String[] GRID_LIGHT_COLORS = {"#7cb5ec", "#f7a35c",
		"#90ee7e", "#7798BF", "#aaeeee", "#ff0066", "#eeaaee", "#55BF3B", "#DF5353", "#7798BF", "#aaeeee"};
	
	/**
	 * Returns the highcharts default color set of a given theme
	 */
	public static String[] getThemeColors(Theme theme) {
		if (Theme.DEFAULT.equals(theme)) {
			return DEFAULT_COLORS;
		} else if (Theme.GRID.equals(theme)) {
			return GRID_COLORS;
		} else if (Theme.SKIES.equals(theme)) {
			return SKIES_COLORS;
		} else if (Theme.GRAY.equals(theme)) {
			return GRAY_COLORS;
		} else if (Theme.DARK_BLUE.equals(theme)) {
			return DARK_BLUE_COLORS;
		} else if (Theme.DARK_GREEN.equals(theme)) {
			return DARK_GREEN_COLORS;
		} else if (Theme.DARK_UNICA.equals(theme)) {
			return DARK_UNICA_COLORS;
		} else if (Theme.SAND_SIGNIKA.equals(theme)) {
			return SAND_SIGNIKA_COLORS;
		} else if (Theme.GRID_LIGHT.equals(theme)) {
			return GRID_LIGHT_COLORS;
		} else {
			return null;
		}
	}
}
