/* Zone.java

	Purpose:
		
	Description:
		
	History:
		Sat, Oct  3, 2015  5:10:31 PM, Created by Christopher

Copyright (C) 2015 Potix Corporation. All Rights Reserved.

This program is distributed under LGPL Version 2.1 in the hope that
it will be useful, but WITHOUT ANY WARRANTY.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * Defining zones within a series
 * @author Christopher
 */
public class Zone extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		color,
		dashStyle,
		fillColor,
		value
	}

	/**
	 * Get the color of the series.
	 * <p>
	 * Default: null
	 * @since 2.1.0
	 */
	public Color getColor() {
		return (Color) getAttr(Attrs.color, null).asValue();
	}
	/**
	 * Set the color of the series.
	 * @since 2.1.0
	 */
	public void setColor(Color color) {
		setAttr(Attrs.color, color, NOT_NULL_VALUE);
	}
	/**
	 * Set the color of the series.
	 * @since 2.1.0
	 */
	public void setColor(String color) {
		setColor(new Color(color));
	}
	/**
	 * Set the color of the series.
	 * @see LinearGradient
	 * @since 2.1.0
	 */
	public void setColor(LinearGradient color) {
		setColor(new Color(color));
	}
	/**
	 * Set the color of the series.
	 * @see RadialGradient
	 * @since 2.1.0
	 */
	public void setColor(RadialGradient color) {
		setColor(new Color(color));
	}
	
	/**
	 * Get name for the dash style to use for the graph.
	 * <p>
	 * Default: null
	 * @since 2.1.0
	 */
	public String getDashStyle() {
		return getAttr(Attrs.dashStyle, null).asString();
	}
	/**
	 * Set name for the dash style to use for the graph.
	 * Possible values are:
	 * <ul>
	 * <li>"Solid"</li>
	 * <li>"ShortDash"</li>
	 * <li>"ShortDot"</li>
	 * <li>"ShortDashDot"</li>
	 * <li>"ShortDashDotDot"</li>
	 * <li>"Dot"</li>
	 * <li>"Dash"</li>
	 * <li>"LongDash"</li>
	 * <li>"DashDot"</li>
	 * <li>"LongDashDot"</li>
	 * <li>"LongDashDotDot"</li>
	 * </ul>
	 * @since 2.1.0
	 */
	public void setDashStyle(String dashStyle) {
		if (!("Solid".equalsIgnoreCase(dashStyle)
				|| "ShortDash".equalsIgnoreCase(dashStyle)
				|| "ShortDot".equalsIgnoreCase(dashStyle)
				|| "ShortDashDot".equalsIgnoreCase(dashStyle)
				|| "ShortDashDotDot".equalsIgnoreCase(dashStyle)
				|| "Dot".equalsIgnoreCase(dashStyle)
				|| "Dash".equalsIgnoreCase(dashStyle)
				|| "LongDash".equalsIgnoreCase(dashStyle)
				|| "DashDot".equalsIgnoreCase(dashStyle)
				|| "LongDashDot".equalsIgnoreCase(dashStyle)
				|| "LongDashDotDot".equalsIgnoreCase(dashStyle)))
			throw new IllegalArgumentException("Unsupported style: ["
					+ dashStyle + "]");
		setAttr(Attrs.dashStyle, dashStyle, "Solid");
	}

	/**
	 * Get the fill color for the series (in area type series).
	 * <p>
	 * Default: null
	 * @since 2.1.0
	 */
	public Color getFillColor() {
		return (Color) getAttr(Attrs.fillColor, null).asValue();
	}
	/**
	 * Set the fill color for the series (in area type series).
	 * @since 2.1.0
	 */
	public void setFillColor(Color fillColor) {
		setAttr(Attrs.fillColor, fillColor, NOT_NULL_VALUE);
	}
	/**
	 * Set the fill color for the series (in area type series).
	 * @since 2.1.0
	 */
	public void setFillColor(String fillColor) {
		setFillColor(new Color(fillColor));
	}
	/**
	 * Set the fill color for the series (in area type series).
	 * @see LinearGradient
	 * @since 2.1.0
	 */
	public void setFillColor(LinearGradient fillColor) {
		setFillColor(new Color(fillColor));
	}
	/**
	 * Set the fill color for the series (in area type series).
	 * @see RadialGradient
	 * @since 2.1.0
	 */
	public void setFillColor(RadialGradient fillColor) {
		setFillColor(new Color(fillColor));
	}
	
	/**
	 * Get value up to where the zone extends, if undefined the zones stretches
	 * to the last value in the series.
	 * <p>
	 * Default: null
	 * @since 2.1.0
	 */
	public Number getValue() {
		return getAttr(Attrs.value, null).asNumber();
	}
	/**
	 * Get value up to where the zone extends, if undefined the zones stretches
	 * to the last value in the series.
	 * @since 2.1.0
	 */
	public void setValue(Number value) {
		setAttr(Attrs.value, value);
	}
}